#! /bin/bash
prog="WDTokenServer_v6"
runshell="wdtokenserver.sh"
FILES_DDIR=/opt/apps/com.watchdata.wdtokenserver/files
PCSC_DDIR=$FILES_DDIR/pcsc

# pcsc daemon名称
DAEMON_NAME=wdpcscd_v6

#枚举应用目录的根路径。在国产系统上握奇UKey应用都装在/opt/apps下
APP_ROOT=/opt/apps
#枚举深度。为处理握奇UKey应用装在app的子路径下的情况
APP_DEPTH=1
#握奇UKey应用info文件名
APP_INFO_FILENAME=wdtokenserverinfo

#终止已运行的pcscd服务
function killpcscd(){
	#终止服务进程
	pidlist=$(ps -ef | grep "$DAEMON_NAME" | grep -v grep | awk '{print $2}')
	if [ -n "$pidlist" ];then
		echo "	$DAEMON_NAME is running, will stop it ..."
		echo $pidlist | while read pid; do
		kill -9 $pid
		done
		sleep 02s
		
		#检查wdpcscd服务进程是否结束
		pidlist=$(ps -ef | grep "$DAEMON_NAME" | grep -v grep | awk '{print $2}')
		if [ -n "$pidlist" ];then
			echo "	can't stop $DAEMON_NAME."
			ErrFail
		else
			echo "	$DAEMON_NAME has been stopped."
		fi
	fi

	#清理
	if [ -e /tmp/$DAEMON_NAME.comm ]; then
		rm -f /tmp/$DAEMON_NAME.comm
	fi
	if [ -e /tmp/$DAEMON_NAME.pid ]; then
		rm -f /tmp/$DAEMON_NAME.pid
	fi
}

#根据传入的InfoPlist文件名，VID和PID，判断现有文件中是否包括指定VID,PID，如包括则回显exist，不包括则什么都不做
function checkexist(){
    infofile=$1
    VID=$2
    PID=$3
    vidfound=false
    while read rows
    do
        echo "current info row=$rows"
        if [ $vidfound ]; then
            result=$(echo $rows | grep "$PID")
            if [[ "$result" != "" ]]; then
                echo "exist"
                break
            fi
        else
            result=$(echo $rows | grep "$VID")
            if [[ "$result" != "" ]]; then
                vidfound=true
                continue
            fi
        fi

    done < $infofile
}

#读取传入的握奇ukey应用文件名，根据其中记载的tokenserver版本，设备type(UDK/CCID)，检查是否已存在，如不存在则将其注册进VID/PID并重启pcscd
function regist_wdapp_if_unexist(){
    input=$1
    echo "regist $input ..."
    while read rows
    do
        echo "current info row=$rows"
        REGISTINFO=${rows#REGISTINFO=}
        if [ "$REGISTINFO" == "" ]; then
            continue
        fi
        type=${REGISTINFO%%,*}
        REGISTINFO=`echo ${REGISTINFO#*,}`
        VID=`echo ${REGISTINFO%%,*}`
        REGISTINFO=`echo ${REGISTINFO#*,}`
        PID=`echo ${REGISTINFO%%,*}`
        INFO=`echo ${REGISTINFO#*,}`
        if [ "$type" == "UDK" ]; then
            result=`checkexist $PCSC_DDIR/drivers/ifd-udk.bundle/Contents/Info.plist $VID $PID`
            echo "checkexist $PCSC_DDIR/drivers/ifd-udk.bundle/Contents/Info.plist $VID $PID=$result"
            if [[ "$result" == "" ]]; then
                echo "$FILES_DDIR/bin/pcsc_drivers_regist $PCSC_DDIR/drivers/ifd-udk.bundle/Contents/Info.plist + $VID $PID $INFO"
                $FILES_DDIR/bin/pcsc_drivers_regist "$PCSC_DDIR/drivers/ifd-udk.bundle/Contents/Info.plist" + $VID $PID "$INFO" 
                
                #kill pcscd
                killpcscd

                RUNProg=`ps -ef | grep ${prog} | grep -v grep | awk '{print $2}'`
                if [ "x${RUNProg}" != "x" ]; then
                echo "WDTokenServer is running. only restart pcsc"
                ${PCSC_DDIR}/${DAEMON_NAME} &

                else
                echo "start WDTokenServer & pcsc"
                ${PCSC_DDIR}/${runshell} &
                fi
            fi
        elif [ "$type" == "CCID" ]; then
            result=`checkexist $PCSC_DDIR/drivers/ifd-ccid.bundle/Contents/Info.plist $VID $PID`
            echo "checkexist $PCSC_DDIR/drivers/ifd-ccid.bundle/Contents/Info.plist $VID $PID=$result"
            if [[ "$result" == "" ]]; then
                echo "$FILES_DDIR/bin/pcsc_drivers_regist $PCSC_DDIR/drivers/ifd-ccid.bundle/Contents/Info.plist + $VID $PID $INFO"
                $FILES_DDIR/bin/pcsc_drivers_regist "$PCSC_DDIR/drivers/ifd-ccid.bundle/Contents/Info.plist" + $VID $PID "$INFO" 
                
                #kill pcscd
                killpcscd

                RUNProg=`ps -ef | grep ${prog} | grep -v grep | awk '{print $2}'`
                if [ "x${RUNProg}" != "x" ]; then
                echo "WDTokenServer is running. only restart pcsc"
                ${PCSC_DDIR}/${DAEMON_NAME} &

                else
                echo "start WDTokenServer & pcsc"
                ${PCSC_DDIR}/${runshell} &
                fi
            fi
        fi
    done < $input
}

lock_file=/tmp/registwdtokenserver.lock

#查询文件锁
while [ -f $lock_file ]
do
    pid=`cat $lock_file`
    echo "$lock_file exist, readed pid=$pid"
    ps $pid &>/dev/null
    if [ $? -eq 0 ]; then
        echo "waiting $lock_file"
        sleep 1s
        continue;
    else
        break;
    fi
done

echo "$lock_file in"

#加锁
echo "add $lock_file"
echo $$ > $lock_file

regist_wdapp_if_unexist $1

sudo chmod 777 /tmp
sudo chmod 777 /tmp -R

#释放锁
rm -rf $lock_file
echo "$lock_file free"